@@ -11,7 +11,6 @@ Log::Handler - Log messages to several outputs.
     $log->add(
         file => {
             filename => "file.log",
-            mode     => "append",
             maxlevel => "debug",
             minlevel => "warning",
         }
@@ -29,6 +28,7 @@ Or
             maxlevel => "debug",
             minlevel => "debug",
             message_layout => "%T [%L] %m (%C)",
+        },
         screen => {
             log_to   => "STDOUT",
             maxlevel => "info",
@@ -41,6 +41,18 @@ Or
         },
     );
 
+Or
+
+    use Log::Handler;
+
+    my $log = Log::Handler->new();
+
+    $log->config( config => "logger.conf" );
+
+    # and maybe later
+
+    $log->reload( config => "logger.conf" );
+
 Or
 
     # init myapp logger with accessor LOG
@@ -61,19 +73,14 @@ Or
     my $log = Log::Handler->get_logger("myapp");
     $log->info("info message from MyApp::Bar");
 
-Or very fast
-
-    use Log::Handler;
-
-    my $log = Log::Handler->new("screen");
-
 =head1 DESCRIPTION
 
 The C<Log::Handler> is a object oriented handler for logging, tracing and
 debugging. It is very easy to use and provides a simple interface for
 multiple output objects with lots of configuration parameters. You can
 easily filter the amount of logged information on a per-output base,
-define priorities and create patterns to format the messages.
+define priorities, create patterns to format the messages and reload
+the complete logging machine.
 
 See the documentation for details.
 
@@ -82,12 +89,16 @@ See the documentation for details.
 Note that the default for option C<newline> is now set to TRUE and newlines
 will be appended automatically to each message if no newline exists.
 
-A long time I have thought about this serious change and have come to
-the decision to move the change.
+A long time I thought about this serious change and have come to
+the decision to change it.
 
 The default for option C<mode> from Log::Handler::Output::File is now
 C<append> and not C<excl> anymore.
 
+The methods C<reload()> and C<validate()> are new since version 0.62.
+I tested it with Screen.pm, File.pm and DBI.pm and it runs fine.
+If you find bugs then open a bug report please :-)
+
 =head1 LOG LEVELS
 
 There are eigth levels available:
@@ -108,6 +119,8 @@ every peep.
 
 =head1 LOG LEVEL METHODS
 
+=head2 Level methods
+
 =over 4
 
 =item B<debug()>
@@ -116,7 +129,7 @@ every peep.
 
 =item B<notice()>
 
-=item B<warning()>
+=item B<warning()>, B<warn()>
 
 =item B<error()>, B<err()>
 
@@ -170,15 +183,15 @@ of C<minlevel> and C<maxlevel> would log the message and FALSE if not.
 
 =over 4
 
-=item C<fatal>, C<is_fatal>
+=item B<fatal>, B<is_fatal>
 
-=item C<trace>
+=item B<trace>
 
-=item C<dump>
+=item B<dump>
 
-=item C<die>
+=item B<die>
 
-=item C<log>
+=item B<log>
 
 =back
 
@@ -196,6 +209,45 @@ Call C<new()> to create a new log handler object.
 
 Call C<add()> to add a new output object.
 
+The method expects 2 parts of options; the options for the handler and
+the options for the output module you want to use. The output modules got it's own
+documentation for all options.
+
+Example:
+
+    use Log::Handler;
+
+    my $log = Log::Handler->new();
+
+    $log->add(
+
+        # Add "file output"
+        file => {
+
+            # handler options (see Log::Handler)
+            timeformat      => "%Y/%m/%d %H:%M:%S",
+            message_layout  => "%T [%L] %S: %m",
+            maxlevel        => "debug",
+            minlevel        => "emergency",
+            die_on_errors   => 1,
+            debug_trace     => 0,
+            debug_mode      => 2,
+            debug_skip      => 0,
+
+            # file options (see Log::Handler::Output::File)
+            filename        => "file.log",
+            filelock        => 1,
+            fileopen        => 1,
+            reopen          => 1,
+            autoflush       => 1,
+            permissions     => "0660",
+            utf8            => 1,
+
+        }
+    );
+
+Take a look to L<Log::Handler::Examples> for more examples.
+
 The following options are possible for the handler:
 
 =over 4
@@ -337,8 +389,8 @@ You can create your own placeholders with the method C<set_pattern()>.
 
 This option is just useful if you want to forward messages to output
 modules that needs the parts of a message as a hash reference - as
-example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>,
-L<Log::Handler::Output::Email> or L<Log::Handler::Output::Screen>.
+example L<Log::Handler::Output::Forward>, L<Log::Handler::Output::DBI>
+L<Log::Handler::Output::Screen>.
 
 The option expects a list of placeholders:
 
@@ -466,6 +518,13 @@ If you set C<die_on_errors> to 0 then you have to controll it yourself.
     # or Log::Handler::errstr()
     # or $Log::Handler::ERRSTR
 
+=item B<remove_on_reload>
+
+This option is set to 1 by default.
+
+Take a look to the decription of the method C<reload> for more
+informations to this option.
+
 =item B<filter_message>
 
 With this option it's possible to set a filter. If the filter is set then
@@ -474,7 +533,6 @@ a code reference or a simple string. Example:
 
     $log->add(file => {
         filename => "file.log",
-        mode     => "append",
         maxlevel => 6,
         filter_message => qr/log this/,
         # or
@@ -489,7 +547,6 @@ If you pass your own code then you have to check the message yourself.
 
     $log->add(file => {
         filename => "file.log",
-        mode     => "append",
         maxlevel => 6,
         filter_message => \&my_filter
     });
@@ -505,7 +562,6 @@ hash reference with the options C<matchN> and C<condition>. Example:
 
     $log->add(file => {
         filename => "file.log",
-        mode     => "append",
         maxlevel => 6,
         filter_message => {
             match1    => "log this",
@@ -659,95 +715,6 @@ This option let skip the C<caller()> informations the count of C<debug_skip>.
 
 =back
 
-=head2 HowTo use add()
-
-The method C<add()> excepts 2 parts of options; the options for the handler and
-the options for the output module you want to use. The output modules got it's own
-documentation for all options. As example if you want to add a file-output then
-take a look into the documentation of L<Log::Handler::Output::File> to see which
-options are available.
-
-There are different ways to add a new output to the handler. One way is to create
-the output object yourself and pass it with the handler options to C<add()>.
-
-Example:
-
-    use Log::Handler;
-    use Log::Handler::Output::File;
-
-    # the handler options - how to handle the output
-    my %handler_options = (
-        timeformat      => "%Y/%m/%d %H:%M:%S",
-        message_layout  => "%T [%L] %S: %m",
-        maxlevel        => "debug",
-        minlevel        => "emergency",
-        die_on_errors   => 1,
-        debug_trace     => 0,
-        debug_mode      => 2,
-        debug_skip      => 0,
-    );
-
-    # the file options - how to handle the file
-    my %file_options = (
-        filename        => "file.log",
-        filelock        => 1,
-        fileopen        => 1,
-        reopen          => 1,
-        mode            => "append",
-        autoflush       => 1,
-        permissions     => "0660",
-        utf8            => 1,
-    );
-
-    # create the file object
-    my $file = Log::Handler::Output::File->new( \%file_options );
-
-    # create a new handler object
-    my $log = Log::Handler->new();
-
-    # now we add the file object to the handler with the handler options
-    $log->add( $file => \%handler_options );
-
-But it can be simplier! You can merge all options and pass them to C<add()>
-in one step, you just need to tell the handler what do you want to add.
-
-    use Log::Handler;
-
-    my $log = Log::Handler->new();
-
-    $log->add(
-
-        # what do you want to add - a file output
-        file => {
-
-            # handler options
-            timeformat      => "%Y/%m/%d %H:%M:%S",
-            message_layout  => "%T [%L] %S: %m",
-            maxlevel        => "debug",
-            minlevel        => "emergency",
-            die_on_errors   => 1,
-            debug_trace     => 0,
-            debug_mode      => 2,
-            debug_skip      => 0,
-
-            # file options
-            filename        => "file.log",
-            filelock        => 1,
-            fileopen        => 1,
-            reopen          => 1,
-            mode            => "append",
-            autoflush       => 1,
-            permissions     => "0660",
-            utf8            => 1,
-
-        }
-    );
-
-The options will be splitted internal and you don't need to split it yourself,
-only if you want to do it yourself.
-
-Take a look to L<Log::Handler::Examples> for more informations.
-
 =head2 output()
 
 Call C<output($alias)> to get the output object that you added with
@@ -787,7 +754,6 @@ write operations.
     $log->add(file => {
         filename      => "file.log",
         maxlevel      => "info",
-        mode          => "append",
         die_on_errors => 0,
     });
 
@@ -812,24 +778,28 @@ With this method it's possible to load your output configuration from a file.
 Or
 
     $log->config(config => {
-        file => {
-            default => {
-                debug_mode    => 2,
-                die_on_errors => 0
+        file => [
+            {
+                alias    => "error_log",
+                filename => "error.log",
+                maxlevel => "warning",
+                minlevel => "emerg",
+                priority => 1
             },
-            error_log => {
-                filename      => "error.log",
-                maxlevel      => "warning",
-                minlevel      => "emerg",
-                priority      => 1
+            {
+                alias    => "common_log",
+                filename => "common.log",
+                maxlevel => "info",
+                minlevel => "emerg",
+                priority => 2
             },
-            common_log => {
-                filename      => "common.log",
-                maxlevel      => "info",
-                minlevel      => "emerg",
-                priority      => 2
-            },
-        }
+        ],
+        screen => {
+            alias    => "screen",
+            maxlevel => "debug",
+            minlevel => "emerg",
+            log_to   => "STDERR",
+        },
     });
 
 The key S<"default"> is used here to define default parameters for all file
@@ -838,6 +808,100 @@ outputs. All other keys (C<error_log>, C<common_log>) are used as aliases.
 Take a look into the documentation of L<Log::Handler::Config> for more
 informations.
 
+=head2 reload()
+
+With the method C<reload()> it's possible to reload the logging
+machine. Just pass the complete new configuration for all outputs,
+it works exaclty like C<config()>.
+
+At first you should know that it's highly recommended to set a alias for
+each output. If you don't set a alias then the logger doesn't know which
+output-objects to reload. If a output-objects doesn't have a alias then
+the objects will be removed and the new configuration will be added.
+
+Example:
+
+logger.conf
+
+    <file>
+        alias    = debug
+        filename = debug.log
+        maxlevel = debug
+        minlevel = emerg
+    </file>
+
+    <file>
+        alias    = common
+        filename = common.log
+        maxlevel = info
+        minlevel = emerg
+    </file>
+
+Load the configuration
+
+    $log->config(config => "logger.conf");
+
+Now change the configuration in logger.conf
+
+    <file>
+        alias    = common
+        filename = common.log
+        maxlevel = notice
+        minlevel = emerg
+    </file>
+
+    <sendmail>
+        alias   = sendmail
+        from    = bar@foo.example
+        to      = foo@bar.example
+        subject = your subject
+    </sendmail>
+
+What happends now...
+
+The file-output with the alias C<debug> will be removed,
+the file-output with the alias C<common> will be
+reloaded and the output with the alias C<sendmail> will be added.
+
+If you don't want that output-objects will be removed
+because they were added internal, then you can set the
+option C<remove_on_reload> to 0.
+
+Example:
+
+    $log->config(config => "logger.conf");
+
+    $log->add(
+        forward => {
+            forward_to => \&my_func,
+            remove_on_reload => 0,
+        }
+    );
+
+The forward-output is not removed after a reload.
+
+=head2 validate()
+
+The method C<validate()> expects the same arguments like C<config()> and C<reload()>.
+
+Maybe you want to validate your options before you pass them to C<config()>
+or C<reload()>.
+
+Example:
+
+    my $log = Log::Handler->new();
+
+    $log->config( config => \%config );
+
+    # and maybe later
+
+    if ( $log->validate( config => \%new_config ) ) {
+        $log->reload( config => \%new_config );
+    } else {
+        warn "unable to reload configuration";
+        warn $log->errstr;
+    }
+
 =head2 set_pattern()
 
 With this option you can set your own placeholders. Example:
@@ -852,7 +916,6 @@ Then you can use this pattern in your message layout:
 
     $log->add(file => {
         filename        => "file.log",
-        mode            => "append",
         message_layout  => "%X %m%N",
     });
 
@@ -913,7 +976,7 @@ Or
 
 Since version C<0.50> it's possible to define a application logger.
 This means to create a C<Log::Handler> object and import it into
-all modules of your program.
+all modules of your project.
 
     use Log::Handler alias => accessor;
 
@@ -959,25 +1022,23 @@ don't want to create an accessor.
     my $log = Log::Handler->get_logger("myapp");
     $log->info("message");
 
-For a little example you can take a look into the examples directory
-of the distribution (examples/logger/).
-
 =head1 EXAMPLES
 
 L<Log::Handler::Examples>
 
 =head1 BENCHMARK
 
-Run examples/benchmark/benchmark.pl on a Dual-Core AMD Opteron(tm) Processor 1212 HE (1000 MHz)
+The benchmark (examples/benchmark/benchmark.pl) runs
+on a Intel Core i7-920 with the following result:
 
-    simple pattern output took     :  2 wallclock secs ( 2.29 usr +  0.02 sys =  2.31 CPU) @ 43290.04/s (n=100000)
-    default pattern output took    :  4 wallclock secs ( 3.36 usr +  0.76 sys =  4.12 CPU) @ 24271.84/s (n=100000)
-    complex pattern output took    :  6 wallclock secs ( 4.85 usr +  0.91 sys =  5.76 CPU) @ 17361.11/s (n=100000)
-    message pattern output took    :  5 wallclock secs ( 4.31 usr +  0.82 sys =  5.13 CPU) @ 19493.18/s (n=100000)
-    suppressed output took         :  0 wallclock secs ( 0.17 usr +  0.00 sys =  0.17 CPU) @ 588235.29/s (n=100000)
-    filtered caller output took    :  5 wallclock secs ( 3.97 usr +  0.81 sys =  4.78 CPU) @ 20920.50/s (n=100000)
-    suppressed caller output took  :  1 wallclock secs ( 1.29 usr +  0.00 sys =  1.29 CPU) @ 77519.38/s (n=100000)
-    filtered messages output took  :  5 wallclock secs ( 3.91 usr +  0.84 sys =  4.75 CPU) @ 21052.63/s (n=100000)
+    simple pattern output took     :  1 wallclock secs ( 1.26 usr +  0.01 sys =  1.27 CPU) @ 78740.16/s (n=100000)
+    default pattern output took    :  2 wallclock secs ( 2.08 usr +  0.15 sys =  2.23 CPU) @ 44843.05/s (n=100000)
+    complex pattern output took    :  4 wallclock secs ( 3.22 usr +  0.23 sys =  3.45 CPU) @ 28985.51/s (n=100000)
+    message pattern output took    :  3 wallclock secs ( 2.72 usr +  0.16 sys =  2.88 CPU) @ 34722.22/s (n=100000)
+    suppressed output took         :  0 wallclock secs ( 0.08 usr +  0.00 sys =  0.08 CPU) @ 1250000.00/s (n=100000)
+    filtered caller output took    :  2 wallclock secs ( 2.10 usr +  0.68 sys =  2.78 CPU) @ 35971.22/s (n=100000)
+    suppressed caller output took  :  1 wallclock secs ( 0.54 usr +  0.00 sys =  0.54 CPU) @ 185185.19/s (n=100000)
+    filtered messages output took  :  3 wallclock secs ( 2.62 usr +  0.08 sys =  2.70 CPU) @ 37037.04/s (n=100000)
 
 =head1 EXTENSIONS
 
@@ -994,6 +1055,7 @@ Prerequisites for all modules:
     POSIX
     Time::HiRes
     Sys::Hostname
+    UNIVERSAL
 
 Recommended modules:
 
@@ -1033,34 +1095,11 @@ If you send me a mail then add Log::Handler into the subject.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2007-2008 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2009 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
-=head1 DISCLAIMER OF WARRANTY
-
-BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
-FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
-OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
-PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
-EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
-ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
-YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
-NECESSARY SERVICING, REPAIR, OR CORRECTION.
-
-IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
-WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
-REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
-LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
-OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
-THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
-RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
-FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
-SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
-SUCH DAMAGES.
-
 =cut
 
 package Log::Handler;
@@ -1068,13 +1107,14 @@ package Log::Handler;
 use strict;
 use warnings;
 use Carp;
-use Params::Validate;
+use Params::Validate qw//;
 use Log::Handler::Output;
 use Log::Handler::Config;
 use Log::Handler::Pattern;
+use UNIVERSAL;
 use base qw(Log::Handler::Levels);
 
-our $VERSION = "0.60";
+our $VERSION = "0.62";
 our $ERRSTR  = "";
 
 # $TRACE and $CALLER_LEVEL are both used as global
@@ -1147,6 +1187,7 @@ our %AVAILABLE_OUTPUTS = (
     dbi      => "Log::Handler::Output::DBI",
     screen   => "Log::Handler::Output::Screen",
     socket   => "Log::Handler::Output::Socket",
+    gearman  => "Log::Handler::Output::Gearman",
 );
 
 # use Log::Handler foo => "LOGFOO", bar => "LOGBAR";
@@ -1202,9 +1243,9 @@ sub new {
 
     my $self = bless {
         priority => PRIORITY,   # start priority
-        levels   => { },        # outputs stored by active levels
-        alias    => { },        # outputs stored by an alias
-        output   => [ ],        # all outputs needed for flush()
+        levels   => { },        # outputs (Output.pm) stored by active levels
+        alias    => { },        # outputs (Output.pm) stored by an alias
+        outputs  => [ ],        # all Output::* objects - for flush()
         pattern  =>             # default pattern
             &Log::Handler::Pattern::get_pattern,
     }, $class;
@@ -1227,48 +1268,21 @@ sub add {
         }
     }
 
-    my $output = $self->_new_output(@_);
-    my $levels = $self->{levels};
+    # At first the config will be splitted into
+    # the package name (Log::Handler::Output::*),
+    # the options for the handler and the options
+    # for the output-module.
+    my ($package, $h_opts, $o_opts) = $self->_split_config(@_);
 
-    # Structure:
-    #   $self->{levels}->{INFO} = [ outputs ordered by priority ]
-    #
-    # All outputs that would log the level INFO will be stored to the
-    # hash-tree $self->{levels}->{INFO}. On this way it's possible
-    # to check very fast if the level is active
-    #
-    #   my $levels = $self->{levels};
-    #   if (exists $levels->{INFO}) { ... }
-    #
-    # and loop over all output objects and pass the message to it.
+    # In the next step the handler options
+    # must be validated.
+    $h_opts = $self->_validate_options($h_opts);
 
-    foreach my $level (keys %{$output->{levels}}) {
-        if ($levels->{$level}) {
-            my @old_order = @{$levels->{$level}};
-            push @old_order, $output;
-            $levels->{$level} = [
-                map  { $_->[0] }
-                sort { $a->[1] <=> $b->[1] }
-                map  { [ $_, $_->{priority} ] } @old_order
-            ];
-        } else {
-            push @{$levels->{$level}}, $output;
-        }
-    }
+    # Create the new output-object.
+    my $output = $self->_new_output($package, $h_opts, $o_opts);
 
-    # Structure:
-    #   $self->{alias}->{$alias} = $output_object
-    #
-    # All outputs with an alias are stored to this hash tree.
-    # Each output can be fetched with output($alias);
-
-    if ($output->{alias}) {
-        my $alias = $output->{alias};
-        $self->{alias}->{$alias} = $output;
-    }
-
-    # save all outputs here
-    push @{$self->{outputs}}, $output;
+    # Add the output $self.
+    $self->_add_output($output);
 
     return 1;
 }
@@ -1279,8 +1293,8 @@ sub config {
     my $config = Log::Handler::Config->config(@_);
 
     # Structure:
-    #   $configs->{file} = [ output configs ];
-    #   $configs->{dbi}  = [ output configs ];
+    #   $config->{file} = [ output config ];
+    #   $config->{dbi}  = [ output config ];
 
     foreach my $type (keys %$config) {
         for my $c (@{$config->{$type}}) {
@@ -1291,6 +1305,109 @@ sub config {
     return 1;
 }
 
+sub validate {
+    my $self   = shift;
+    my $class  = ref($self);
+    my @v_opts = (); # validated options
+
+    eval {
+        my $config = Log::Handler::Config->config(@_);
+
+        foreach my $type (keys %$config) {
+            foreach my $output_config (@{ $config->{$type} }) {
+                my ($package, $h_opts, $o_opts) = $self->_split_config($type, $output_config);
+                $h_opts = $self->_validate_options($h_opts);
+                $o_opts = $package->validate($o_opts) or die $package->errstr;
+                push @v_opts, { p => $package, h => $h_opts, o => $o_opts, n => $output_config };
+            }
+        }
+    };
+
+    if ($@) {
+        return $self->_raise_error($@);
+    }
+
+    return \@v_opts;
+}
+
+sub reload {
+    my $self  = shift;
+    my $class = ref($self);
+    my $opts  = $self->validate(@_);
+
+    if (!$opts) {
+        return undef;
+    }
+
+    # Store all aliases that were reloaded or added,
+    # because all output-objects that weren't reloaded
+    # should be removed.
+    my %reloaded = ();
+
+    # Reload in a eval block to prevent that the
+    # program dies - daemons shouldn't die :-)
+    eval {
+        foreach my $output_config (@$opts) {
+            my $package = $output_config->{p}; # package name like Log::Handler::Output::File
+            my $h_opts  = $output_config->{h}; # handler options to reload
+            my $o_opts  = $output_config->{o}; # output options to reload
+            my $n_opts  = $output_config->{n}; # add a new output
+            my $alias   = $h_opts->{alias};
+
+            $reloaded{$alias} = 1;
+
+            # If the alias doesn't exists then a new
+            # output-objects is created, otherwise the
+            # output-object is reloaded.
+            if (!$self->output($alias)) {
+                # If the alias does not exists we use
+                # the alias that was generated on validate().
+                if (!exists $n_opts->{alias}) {
+                    $n_opts->{alias} = $h_opts->{alias};
+                }
+                # Add the new output to Log::Handler
+                $self->add($package => $n_opts);
+            } else {
+                $self->{alias}->{$alias}->reload($h_opts);
+                $self->output($alias)->reload($o_opts)
+                    or die $self->output($alias)->errstr;
+            }
+        }
+    };
+
+    # Something wents wrong.
+    # The error message should be in $ERRSTR.
+    if ($@) {
+        return $self->_raise_error($@);
+    }
+
+    # Rebuild the arrays...
+    $self->{levels}  = { };
+    $self->{outputs} = [ ];
+
+    foreach my $alias (keys %{ $self->{alias} }) {
+        my $output = $self->{alias}->{$alias};
+
+        # Delete all objects that wasn't reloaded and have
+        # set the flag "remove_on_reload".
+
+        if (!exists $reloaded{$alias} && $output->{remove_on_reload}) {
+            # At this point the output object should be destroyed,
+            # because the last reference was stored here.
+            eval { delete $self->{alias}->{$alias} };
+
+            if ($@) {
+                warn $@;
+            }
+        } else {
+            # At this point the output object should be destroyed,
+            $self->_add_output($output);
+        }
+    }
+
+    return 1;
+}
+
 sub set_pattern {
     (@_ == 3 || @_ == 4)
         or Carp::croak 'Usage: $log->set_pattern( $pattern, $name, $code )';
@@ -1391,7 +1508,8 @@ sub flush {
     if (@alias) {
         foreach my $name (@alias) {
             my $output = $self->output($name);
-            next unless $output;
+            next unless $output && UNIVERSAL::can($output, "flush");
+
             if ( !$output->flush ) {
                 if ( defined $errors ) {
                     $errors .= "; " . $output->errstr;
@@ -1402,6 +1520,8 @@ sub flush {
         }
     } else {
         foreach my $output (@{$self->{outputs}}) {
+            next unless UNIVERSAL::can($output, "flush");
+
             if ( !$output->flush ) {
                 if ( defined $errors ) {
                     $errors .= "; " . $output->errstr;
@@ -1415,12 +1535,52 @@ sub flush {
     return defined $errors ? $self->_raise_error($errors) : 1;
 }
 
-sub errstr { $ERRSTR }
+sub errstr {
+    return $ERRSTR;
+}
 
 #
 # private stuff
 #
 
+sub _split_config {
+    my $self = shift;
+    my $type = shift;
+    my $args = shift || { };
+    my $package = ();
+
+    # Split the handler and output options from $args.
+    my ($handler_opts, $output_opts) = $self->_split_options($args);
+
+    # Try to determine which output is wanted...
+    if (exists $AVAILABLE_OUTPUTS{$type}) {
+        $package = $AVAILABLE_OUTPUTS{$type};
+    } elsif ($type =~ /::/) {
+        $package = $type;
+    } else {
+        $package = "Log::Handler::Output::" . ucfirst($type);
+    }
+
+    eval "require $package";
+
+    if ($@) {
+        Carp::croak($@);
+    }
+
+    return ($package, $handler_opts, $output_opts);
+}
+
+sub _new_output {
+    my ($self, $package, $h_opts, $o_opts) = @_;
+
+    my $o_obj = $package->new($o_opts)
+        or Carp::croak $package->errstr;
+
+    my $o_main_obj = Log::Handler::Output->new($h_opts, $o_obj);
+
+    return $o_main_obj;
+}
+
 sub _split_options {
     my ($self, $opts) = @_;
     my (%handler_opts, %output_opts);
@@ -1449,6 +1609,7 @@ sub _split_options {
         newline
         priority
         timeformat
+        remove_on_reload
     );
 
     foreach my $key (keys %$opts) {
@@ -1462,58 +1623,56 @@ sub _split_options {
     return (\%handler_opts, \%output_opts);
 }
 
-sub _new_output {
-    my $self    = shift;
-    my $type    = shift;
-    my $args    = @_ > 1 ? {@_} : shift;
-    my $package = ref($type);
-    my ($output, $handler_opts, $output_opts);
+sub _add_output {
+    my ($self, $output) = @_;
+    my $levels = $self->{levels};
 
-    # add("screen")
-    $args ||= { };
-
-    # There are two ways to add an output:
+    # Structure:
+    #   $self->{levels}->{INFO} = [ outputs ordered by priority ]
     #
-    #   my $log  = Log::Handler->new();
-    #   my $file = Log::Handler::Output::File->new(\%output_opts);
-    #   $log->add($file => \%handler_opts);
+    # All outputs that would log the level INFO will be stored to the
+    # hash-tree $self->{levels}->{INFO}. On this way it's possible
+    # to check very fast if the level is active
     #
-    # and
+    #   my $levels = $self->{levels};
+    #   if (exists $levels->{INFO}) { ... }
     #
-    #   my $log = Log::Handler->new();
-    #   $log->add(file => \%all_options);
+    # and loop over all output objects and pass the message to it.
 
-    if ( length($package) ) {
-        $output = $type;
-        $handler_opts = $args;
-    } else {
-        # Split the handler and output options from $args.
-        ($handler_opts, $output_opts) = $self->_split_options($args);
-
-        # Try to determine which output is wanted...
-        if (exists $AVAILABLE_OUTPUTS{$type}) {
-            $package = $AVAILABLE_OUTPUTS{$type};
-        } elsif ($type =~ /::/) {
-            $package = $type;
+    foreach my $level (keys %{$output->{levels}}) {
+        if ($levels->{$level}) {
+            my @old_order = @{$levels->{$level}};
+            push @old_order, $output;
+            $levels->{$level} = [
+                map  { $_->[0] }
+                sort { $a->[1] <=> $b->[1] }
+                map  { [ $_, $_->{priority} ] } @old_order
+            ];
         } else {
-            $package = "Log::Handler::Output::" . ucfirst($type);
+            push @{$levels->{$level}}, $output;
         }
-
-        eval "require $package";
-        die $@ if $@;
-
-        $output = $package->new($output_opts)
-            or Carp::croak $package->errstr;
     }
 
-    $handler_opts = $self->_validate_options($handler_opts);
-    return Log::Handler::Output->new($handler_opts, $output);
+    # Structure:
+    #   $self->{alias}->{$alias} = $output_object
+    #
+    # All outputs with an alias are stored to this hash tree.
+    # Each output can be fetched with output($alias);
+
+    if ($output->{alias}) {
+        my $alias = $output->{alias};
+        $self->{alias}->{$alias} = $output;
+    }
+
+    # save all outputs here
+    push @{$self->{outputs}}, $output;
 }
 
 sub _validate_options {
     my ($self, @args) = @_;
-    my %wanted  = ();
     my $pattern = $self->{pattern};
+    my $alias   = $self->{alias};
+    my %wanted  = ();
 
     # Option "filter" is deprecated.
     if (exists $args[0]{filter}) {
@@ -1601,8 +1760,25 @@ sub _validate_options {
             type => Params::Validate::SCALAR | Params::Validate::SCALARREF,
             optional => 1,
         },
+        remove_on_reload => {
+            type => Params::Validate::SCALAR,
+            default => 1,
+        },
     });
 
+    if (!$options{alias}) {
+        for (;;) {
+            my $rand = rand();
+
+            if (exists $alias->{$rand}) {
+                next;
+            }
+
+            $options{alias} = $rand;
+            last;
+        }
+    }
+
     if ($options{filter_message}) {
         $options{filter_message} = $self->_validate_filter($options{filter_message});
     }
